<?php
/**
 *
 * WARNING: Please do not edit this file.
 * @see http://codex.wordpress.org/Child_Themes
 *
 * Load the theme function files (options panel, theme functions, widgets, etc...).
 */

include_once get_template_directory() . '/theme/class-symphony.php'; // Symphony Class (main functionality, actions/filters)
include_once get_template_directory() . '/theme/class-symphony-customizer.php'; // Symphony Customizer Class (specific to the customizer)

include_once get_template_directory() . '/includes/class-tgm-plugin-activation.php'; // TGM Activation

include_once get_template_directory() . '/includes/theme-options.php'; // SDS Theme Options
include_once get_template_directory() . '/includes/theme-functions.php'; // SDS Theme Options Functions
include_once get_template_directory() . '/includes/class-customize-us-control.php'; // Customize Controller
include_once get_template_directory() . '/includes/widget-social-media.php'; // SDS Social Media Widget=

/**
 * This function registers all content layouts available in this theme.
 */
if ( ! function_exists( 'sds_content_layouts' ) ) {
	function sds_content_layouts() {
		$content_layouts = array(
			'default' => array( // Name used in saved option
				'label' => __( 'Default', 'symphony' ), // Label on options panel (required)
				'preview' => '<div class="cols cols-1 cols-default"><div class="col col-content" title="%1$s"><span class="label">%1$s</span></div></div>', // Preview on options panel (required; %1$s is replaced with values below on options panel if specified)
				'preview_values' => array( __( 'Default', 'symphony' ) ),
				'default' => true
			),
			'cols-1' => array( // Full Width
				'label' => __( 'Full Width', 'symphony' ),
				'preview' => '<div class="cols cols-1"><div class="col col-content"></div></div>',
			),
			'cols-2' => array( // Content Left, Primary Sidebar Right
				'label' => __( 'Content Left', 'symphony' ),
				'preview' => '<div class="cols cols-2"><div class="col col-content"></div><div class="col col-sidebar"></div></div>'
			),
			'cols-2-r' => array( // Content Right, Primary Sidebar Left
				'label' => __( 'Content Right', 'symphony' ),
				'preview' => '<div class="cols cols-2 cols-2-r"><div class="col col-sidebar"></div><div class="col col-content"></div></div>'
			),
			'cols-3' => array( // Content Left, Primary Sidebar Middle, Secondary Sidebar Right
				'label' => __( 'Content, Sidebar, Sidebar', 'symphony' ),
				'preview' => '<div class="cols-3"><div class="col col-content"></div><div class="col col-sidebar"></div><div class="col col-sidebar col-sidebar-secondary"></div></div>'
			),
			'cols-3-m' => array( // Primary Sidebar Left, Content Middle, Secondary Sidebar Right
				'label' => __( 'Sidebar, Content, Sidebar', 'symphony' ),
				'preview' => '<div class="cols cols-3 cols-3-m"><div class="col col-sidebar"></div><div class="col col-content"></div><div class="col col-sidebar col-sidebar-secondary"></div></div>'
			),
			'cols-3-r' => array( // Primary Sidebar Left, Secondary Sidebar Middle, Content Right
				'label' => __( 'Sidebar, Sidebar, Content', 'symphony' ),
				'preview' => '<div class="cols cols-3 cols-3-r"><div class="col col-sidebar"></div><div class="col col-sidebar col-sidebar-secondary"></div><div class="col col-content"></div></div>'
			)
		);

		return apply_filters( 'sds_theme_options_content_layouts', $content_layouts );
	}
}

/**
 * This function modifies the global $content_width value based on content layout or page template settings.
 */
if ( ! function_exists( 'symphony_body_class' ) ) {
	add_filter( 'body_class', 'symphony_body_class', 20 );

	function symphony_body_class( $classes ) {
		global $sds_theme_options, $content_width;

		// Symphony Customizer
		$symphony_customizer= Symphony_Customizer_Instance();

		// Content layout was specified by user in Theme Options
		if ( isset( $sds_theme_options['body_class'] ) && ! empty( $sds_theme_options['body_class'] ) ) {
			// 1 Column
			if ( $sds_theme_options['body_class'] === 'cols-1' )
				$content_width = 1600;
			// 3 Columns
			else if ( strpos( $sds_theme_options['body_class'], 'cols-3' ) !== false )
				$content_width = 722;
		}

		// Page Template was specified by the user for this page
		if ( ! empty( $sds_theme_options['page_template'] ) && $sds_theme_options['page_template'] !== 'default' ) {
			// Full Width or Landing Page
			if( in_array( $sds_theme_options['page_template'], array( 'template-full-width.php', 'template-landing-page.php' ) ) )
				$content_width = 1600;
		}

		// Customizer
		if ( $symphony_customizer->is_customize_preview() )
			$classes['symphony-customizer'] = 'customizer';

		return $classes;
	}
}

/**
 * This function sets a default featured image size for use in this theme.
 */
if ( ! function_exists( 'sds_theme_options_default_featured_image_size' ) ) {
	add_filter( 'sds_theme_options_default_featured_image_size', 'sds_theme_options_default_featured_image_size' );

	function sds_theme_options_default_featured_image_size( $default ) {
		return 'symphony-1600x9999';
	}
}

/**
 * This function returns the more link label for Symphony.
 */
function symphony_more_link_label( $return_default_only = false ) {
	// Return default
	if ( $return_default_only )
		return __( 'Continue Reading', 'symphony' );

	// Get theme mod
	$label = get_theme_mod( 'symphony_more_link_label' );

	return ( ! empty( $label ) ) ? $label : __( 'Continue Reading', 'symphony' );
}

/**
 * This function determines if the current theme mods are set to a "fixed width" layout.
 */
function symphony_is_fixed_width() {
	// Return opposite of "symphony_fluid_width" theme mod
	return ! ( bool ) get_theme_mod( 'symphony_fluid_width', true );
}

/**
 * This function determines if the current theme mods are set to a "fluid width" layout.
 */
function symphony_is_fluid_width() {
	return ( bool ) get_theme_mod( 'symphony_fluid_width', true );
}

/*
 * This function determines if Jetpack exists and if the Custom Content Types module is active.
 */
function symphony_jetpack_portfolio_active() {
	return class_exists( 'Jetpack' ) && Jetpack::is_module_active( 'custom-content-types' );
}

/**
 * Filters the blog title tag
 *
 * @param $tag
 *
 * @return string
 */
function sds_logo_filter( $tag ) {
	return 'h1';
}

add_filter( 'sds_logo_wrapper_element', 'sds_logo_filter' );

/**
 * Filters the blog tagline tag
 *
 * @param $tag
 *
 * @return string
 */
function sds_tagline_filter( $tag ) {
	return 'h2';
}

add_filter( 'sds_tagline_wrapper_element', 'sds_tagline_filter' );

/**
 * This function is a conditional that checks to see if the author meta should be shown on the front-end based on settings.
 */
function symphony_show_author_meta() {
	global $sds_theme_options;

	return ! $sds_theme_options['hide_author_meta'];
}

/**
 * This function is a conditional that checks to see if the post meta should be shown on the front-end based on settings.
 */
function symphony_show_post_meta() {
	global $sds_theme_options;

	return ! $sds_theme_options['hide_post_meta'];
}

/**
 * This function returns the Boolean value of the parameter passed.
 */
if ( ! function_exists( 'symphony_boolval' ) ) {
	function symphony_boolval( $var, $wp_customize_setting = false ) {
		return ( bool ) $var;
	}
}

if ( ! function_exists( 'sds_theme_options_ads' ) ) {
	add_action( 'sds_theme_options_ads', 'sds_theme_options_ads' );

	function sds_theme_options_ads() {
	?>
		<div class="sds-theme-options-ad">
			<a href="<?php echo esc_url( sds_get_pro_link( 'theme-options-ad' ) ); ?>" target="_blank" class="sds-theme-options-upgrade-ad">
				<h3><?php _e( 'Upgrade to Symphony Pro!', 'symphony' ); ?></h3>
				<ul>
					<li><?php _e( 'Priority Ticketing Support', 'symphony' ); ?></li>
					<li><?php _e( 'More Customizer Settings', 'symphony' ); ?></li>
					<li><?php _e( 'Conductor Support', 'symphony' ); ?></li>
					<li><?php _e( 'Easily Add Custom Scripts/Styles', 'symphony' ); ?></li>
					<li><?php _e( 'and More!', 'symphony' ); ?></li>
				</ul>

				<span class="sds-theme-options-btn-green"><?php _e( 'Upgrade Now!', 'symphony' ); ?></span>
			</a>
		</div>

		<div class="sds-theme-options-ad">
			<a href="<?php echo esc_url( 'http://conductorplugin.com/slocum-themes/' ); ?>" target="_blank" class="sds-theme-options-upgrade-ad sds-theme-options-upgrade-ad-dark-gray">
				<h3><?php _e( 'Introducing Conductor Plugin', 'symphony' ); ?></h3>
				<ul>
					<li><?php _e( 'Custom Layouts', 'symphony' ); ?></li>
					<li><?php _e( 'Custom Content Displays', 'symphony' ); ?></li>
					<li><?php _e( 'No Code Required!', 'symphony' ); ?></li>
				</ul>

				<span class="sds-theme-options-btn-yellow"><?php _e( 'Get Conductor!', 'symphony' ); ?></span>
			</a>
		</div>
	<?php
	}
}

if ( ! function_exists( 'sds_theme_options_upgrade_cta' ) ) {
	add_action( 'sds_theme_options_upgrade_cta', 'sds_theme_options_upgrade_cta' );

	function sds_theme_options_upgrade_cta( $type ) {
		switch( $type ) :
			case 'color-schemes':
		?>
				<p>
					<?php
						printf( '<a href="%1$s" target="_blank">%2$s</a> %3$s',
							esc_url( sds_get_pro_link( 'theme-options-colors' ) ),
							__( 'Upgrade to Symphony Pro', 'symphony' ),
							__( 'and receive more color schemes!', 'symphony' )
						);
					?>
				</p>
		<?php
			break;
			case 'web-fonts':
		?>
				<p>
					<?php
						printf( '<a href="%1$s" target="_blank">%2$s</a> %3$s',
							esc_url( sds_get_pro_link( 'theme-options-fonts' ) ),
							__( 'Upgrade to Symphony Pro', 'symphony' ),
							__( 'to use more web fonts!', 'symphony' )
						);
					?>
				</p>
		<?php
			break;
			case 'help-support':
		?>
				<p>
					<?php
						printf( '<a href="%1$s" target="_blank">%2$s</a> %3$s',
							esc_url( sds_get_pro_link( 'theme-options-help' ) ),
							__( 'Upgrade to Symphony Pro', 'symphony' ),
							__( 'to receive priority ticketing support!', 'symphony' )
						);
					?>
				</p>
		<?php
			break;
		endswitch;
	}
}

function sds_get_pro_link( $content = '' ) {
	return esc_url( 'https://slocumthemes.com/wordpress-themes/symphony-pro/#purchase-theme' );
}

if ( ! function_exists( 'sds_theme_options_help_support_tab_content' ) ) {
	add_action( 'sds_theme_options_help_support_tab_content', 'sds_theme_options_help_support_tab_content' );

	function sds_theme_options_help_support_tab_content( ) {
	?>
		<p><?php printf( __( 'If you\'d like to create a support request, please visit the <a href="%1$s">Symphony Forums on WordPress.org</a>.', 'symphony' ), esc_url( 'http://wordpress.org/support/theme/symphony/' ) ); ?></p>
	<?php
	}
}

if ( ! function_exists( 'sds_copyright_branding' ) ) {
	add_filter( 'sds_copyright_branding', 'sds_copyright_branding', 10, 2 );

	function sds_copyright_branding( $text, $theme_name ) {
		return sprintf( __( '<a href="%1$s">%2$s by Slocum Studio</a>', 'symphony' ), esc_url( 'https://slocumthemes.com/wordpress-themes/symphony-free/' ), $theme_name );
	}
}